<?php
/**
 * @package Date Manager
 *
 * Copyright 2010 Bob Britton (email: bob@pro-autocare.com)
 */

/*
Plugin Name: Date Manager
Description: Date Manager Plugin
Author: Todd Crowe
Version: 1.0.5
Author URI: http://www.toddcrowe.com
*/

$bobb_dates_plugin_name = __('Date Manager', 'bobb_dates_trans_domain');
$bobb_dates_plugin_name_full = __('Date Manager Plugin', 'bobb_dates_trans_domain');
$bobb_dates_config_name = $bobb_dates_plugin_name . " " . __('Options', 'bobb_dates_trans_domain');

global $wpdb;
global $bobb_dates_dbversion, $bobb_dates_data_tbl;
$bobb_dates_dbversion = "1.0";
$bobb_dates_data_tbl = $wpdb->prefix . "bobb_dates_data";

register_activation_hook(bobb_dates_get_plugin_file(), 'bobb_dates_install');
if (is_admin()) {
    add_action('admin_menu', 'bobb_dates_add_pages');
    add_action('admin_init', 'bobb_dates_init');
}
add_action('parse_request', 'bobb_dates_parse_request');
add_filter('query_vars', 'bobb_dates_query_vars');

function bobb_dates_install() {
	//http://codex.wordpress.org/Creating_Tables_with_Plugins
	global $wpdb, $bobb_dates_data_tbl, $bobb_dates_dbversion;

	require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

	if($wpdb->get_var("SHOW TABLES LIKE '$bobb_dates_data_tbl'") != $bobb_dates_data_tbl) {
		$sql = "CREATE TABLE " . $bobb_dates_data_tbl . " (
			data_id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
			data_name VARCHAR(128) NOT NULL,
			data_handle VARCHAR(128) NOT NULL,
			data_list TEXT NOT NULL,
			UNIQUE KEY data_id (`data_id`)
			);";
		dbDelta($sql);
	}

	add_option("bobb_dates_dbversion", $bobb_dates_dbversion);
}

function bobb_dates_add_pages() {
    global $bobb_dates_config_name, $bobb_dates_plugin_name;

    add_menu_page($bobb_dates_plugin_name, 'Date Manager', 'administrator', 'bobb_dates_menu', 'bobb_dates_config_options');
    add_submenu_page('bobb_dates_menu', 'Configuration', 'Configuration', 'administrator', 'bobb_dates_menu', 'bobb_dates_config_options');
    add_submenu_page('bobb_dates_menu', 'Dates URLs', 'Dates URLs', 'administrator', 'bobb_dates_menu2', 'bobb_dates_config_dates');
}

function bobb_dates_init() {
    if (function_exists('current_user_can') && current_user_can('manage_options')) {
    	register_setting('bobb_dates_options', 'bobb_dates_api_name');
    	register_setting('bobb_dates_options', 'bobb_dates_api_key');
    }
}

function bobb_dates_get_plugin_dir()
{
	return ABSPATH.PLUGINDIR.'/bobb_dates/';
}

function bobb_dates_get_plugin_dir_url()
{
	return WP_PLUGIN_URL.'/'.str_replace(basename(__FILE__),"",plugin_basename(__FILE__)); 
}

function bobb_dates_get_plugin_file()
{
	// Special magic sauce for enabling activation hook to work with symbolic links
	//  -- Thanks to Amazon Widgets Shortcodes plugin
	if (function_exists('is_link') && is_link(ABSPATH.PLUGINDIR.'/bobb_lander')) {
		return bobb_dates_get_plugin_dir().basename(__FILE__);
	} else {
		return __FILE__;
	}
}


/* bobb_dates_config_options
 */
function bobb_dates_config_options() {
	global $bobb_dates_plugin_name_full;

	$err = $msg = '';

	?>
<div class="wrap">
  <h2><?php echo $bobb_dates_plugin_name_full ?></h2>

  <?php if (!empty($err)) { ?>
  <p style="color: #C00; margin: 4px 0px 4px 24px">Error: <?php echo $err ?></p>
  <?php } else if (!empty($msg)) { ?>
  <p style="color: #0A0; margin: 4px 0px 4px 24px"><?php echo $msg ?></p>
  <?php } ?>

  <form name="bobb_dates_options_form" method="post" action="options.php">
	<table class="form-table">
	<tr valign="top">
	  <th scope="row"><?php _e("Infusionsoft Application Name:", 'bobb_dates_trans_domain' ); ?></th>
	  <td><input type="text" name="bobb_dates_api_name" value="<?php echo get_option('bobb_dates_api_name'); ?>" style="width:95%" /></td>
	  <td>(e.g. myapp.infusionsoft.com)</td>
	</tr>
	<tr valign="top">
	  <th scope="row"><?php _e("Infusionsoft Application Key:", 'bobb_dates_trans_domain' ); ?></th>
	  <td><input type="text" name="bobb_dates_api_key" value="<?php echo get_option('bobb_dates_api_key'); ?>" style="width:95%" /></td>
	  <td>(e.g. 21ac2ccd557f27b139842ac24b893b91)</td>
	</tr>
    </table>
	<?php settings_fields('bobb_dates_options'); ?>
	<p class="submit">
	<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
	</p>
  </form>
</div>

<?php
}

/* bobb_dates_config_businesses
 */
function bobb_dates_config_dates() {
	global $wpdb, $bobb_dates_data_tbl;
	global $bobb_dates_plugin_name, $bobb_dates_plugin_name_full;

	$err = '';
	$view = 'View';

	$data_id = isset($_POST['bobb_dates_data_id']) ? intval($_POST['bobb_dates_data_id']) : 0;

	if (isset($_POST['DoEdit'])) {
		$isApp = bobb_dates_init_infusionsoft(get_option('bobb_dates_api_name'), get_option('bobb_dates_api_key'));
		if ($isApp) {
			$fields = bobb_dates_get_infusionsoft_fields($isApp);
		} else {
			$err .= "Unable to connect to Infusionsoft.<br/>";
		}

		if ($data_id) {
			$view = 'Edit';
			$data = $wpdb->get_row("SELECT * FROM $bobb_dates_data_tbl WHERE data_id='$data_id'", ARRAY_A);
			$data['data_list'] = unserialize($data['data_list']);
			$url = get_bloginfo('url')."/index.php?bobb_dates=".bobb_dates_get_url_handle($data['data_name']);
		} else {
			$view = 'Add';
			$data = array('data_name'=>'', 'data_list'=>array('field'=>array(),'format'=>array()));
			$url = '';
		}
	} else if (isset($_POST['Edit'])) {
		$data_name = $_POST['bobb_dates_data_name'];
		$data_list = $_POST['bobb_dates_data_list'];

		if (empty($data_name)) {
			$err .= "You must provide a name for each date URL.<br/>";
        }
		if (count($data_list['field']) < 2) {
			$err .= "You must specify at least one field/format for each date URL.<br/>";
		}

		$data_handle = bobb_dates_get_url_handle($data_name);

		foreach ($data_list as $key=>$data) {
			unset($data_list[$key][0]);
			$data_list[$key] = array_values($data_list[$key]);
		}
		for ($i = 0; $i < count($data_list['delta_qty']); $i++) {
			if (strcmp($data_list['delta_unit'][$i], 'today') === 0 || strpos($data_list['delta_unit'][$i], 'next ') === 0) {
				$data_list['delta'][$i] = $data_list['delta_unit'][$i];
			} else {
				if ($data_list['delta_qty'][$i] >= 0) {
					$data_list['delta'][$i] = '+' . $data_list['delta_qty'][$i] . ' ' . $data_list['delta_unit'][$i];
				} else {
					$data_list['delta'][$i] = $data_list['delta_qty'][$i] . ' ' . $data_list['delta_unit'][$i];
				}
			}
		}
		unset($data_list['delta_qty']);
		unset($data_list['delta_unit']);

		if (empty($err)) {
			if ($data_id == 0) {
				$wpdb->query("INSERT INTO " . $bobb_dates_data_tbl. " SET data_name='$data_name', data_handle='$data_handle', data_list='".serialize($data_list)."'");
			} else {
				$wpdb->query("UPDATE " . $bobb_dates_data_tbl. " SET data_name='$data_name', data_handle='$data_handle', data_list='".serialize($data_list)."'  WHERE data_id='$data_id'");
			}
		}
	} else if (isset($_POST['Delete']) && $data_id != 0) {
		$wpdb->query("DELETE FROM " . $bobb_dates_data_tbl . " WHERE data_id='$data_id'");
	}

	$dataList = $wpdb->get_results("SELECT data_id, data_name FROM $bobb_dates_data_tbl WHERE 1 ORDER BY data_name");
	$select_size = max(3, min(8, count($default_templates)));

	?>
<div class="wrap">
  <h2><?php echo $bobb_dates_plugin_name_full ?></h2>
  <?php if (!empty($err)) { ?>
  <dir style="background-color: #E5B7B7; border: 1px solid #BB2727; color: #BB2727; margin-bottom: 8px; padding: 4px;width: 95%;"><?php echo $err ?></div>
  <?php } ?>
  <form name="bobb_dates_data_form" method="post" data">
	<?php if ($view == 'Add' || $view == 'Edit') { ?>
	<input type="hidden" name="bobb_dates_data_id" value="<?php echo $data_id ?>" />
	<input type="hidden" id="bobb-dates-next-field-id" value="<?php echo min(1,count($data['data_list']['field'])) ?>" />
	<table id="bobb-dates-url-table" class="form-table">
	<tr valign="top">
	  <th scope="row" width="25%"><?php _e("Date Name:", 'bobb_dates_trans_domain'); ?></th>
	  <td width="60%"><input type="text" name="bobb_dates_data_name" value="<?php echo $data['data_name'] ?>" /></td>
	  <td width="15%">&nbsp;</td>
	</tr>
	<?php 
		$i = 0;

		echo bobb_dates_get_field_row($fields, 'TEMPLATE', '', '', '', 'style="display:none"');

		if (count($data['data_list']['field']) > 0) {
			for (; $i < count($data['data_list']['field']); $i++) {
				echo bobb_dates_get_field_row($fields, $i, $data['data_list']['field'][$i], $data['data_list']['delta'][$i], $data['data_list']['format'][$i]);
			}
		} else {
			echo bobb_dates_get_field_row($fields, $i, '0');
		}
	?>
	</table><br/>
	<input type="submit" class="button-primary" name="Edit" value="<?php echo "$view Date Entry" ?>" />
	<input type="button" class="button-primary" id="bobb-dates-add-field" value="Add Date Field" />
	<input type="submit" class="button-primary" name="Cancel" value="Cancel" />
    <script type="text/javascript">
    var regex = new RegExp('TEMPLATE', 'g');
    jQuery.fn.outerHtml = function(s) { return (s) ? this.before(s).remove() : jQuery("<p>").append(this.eq(0).clone()).html(); };
    jQuery('#bobb-dates-add-field').click(function(){
      var next_id = jQuery('#bobb-dates-next-field-id').val();
	  jQuery('#bobb-dates-next-field-id').val((next_id|0)+1);
	  var html = jQuery('#bobb-dates-field-TEMPLATE').outerHtml().replace(regex, next_id);
      jQuery('#bobb-dates-url-table > tbody').append(html);
      jQuery('#bobb-dates-field-'+next_id).show();
    });
    jQuery('.bobb-dates-del-field').live('click', function(){
      var field = jQuery(this).attr('id').substr(21);
      jQuery('#bobb-dates-field-'+field).remove();
    });
    </script>
	<?php if (!empty($url)) { ?>
    <br/><br/><hr/><br/>
	<p>Send 'HTTP Post' to this address:</p>
    <input type="text" value="<?php echo $url ?>" readonly style="width:95%" />
	<?php } ?>
	<?php } else { ?>
	<table class="form-table">
	<tr valign="top">
	  <th scope="row"><?php _e("Date URL:", 'bobb_dates_trans_domain'); ?></th>
	  <td>
        <select name="bobb_dates_data_id" size="<?php echo $select_size ?>" style="height:inherit;width:50%">
	    <?php foreach ($dataList as $data) { ?>
	    <option value="<?php echo $data->data_id ?>"><?php echo $data->data_name . ' (' . bobb_dates_get_url_handle($data->data_name) . ')' ?></option>
	    <?php } ?>
	    </select>
      </td>
    </tr>
	</table>
	<p class="submit">
      <input type="submit" class="button-primary" name="DoEdit" value="<?php _e('Add/Edit URL') ?>" />
	  <input type="submit" class="button-primary" name="Delete" value="<?php _e('Delete URL') ?>" />
    </p>
    <br/><br/><hr/><br/>
	<p>Send 'HTTP Post' to this address:</p>
    <input type="text" value="<?php echo get_bloginfo('url')."/index.php?bobb_dates=(handle)" ?>" readonly style="width:95%" />
	<?php } ?>
  </form>
</div>

<?php
}


function bobb_dates_parse_request($wp) {
	global $wpdb, $bobb_dates_data_tbl;

	if (array_key_exists('bobb_dates', $wp->query_vars)) {
		$contact_id = intval($_REQUEST['Id']);
		if (!$contact_id) $contact_id = intval($_REQUEST['ContactId']);
		if (!$contact_id) {
			echo 'Invalid arguments.';
			exit;
		}

		$handle = $wp->query_vars['bobb_dates'];
		$data = $wpdb->get_row("SELECT * FROM $bobb_dates_data_tbl WHERE data_handle='$handle'", ARRAY_A);
		if ($data) {
			$data['data_list'] = unserialize($data['data_list']);
		} else {
			echo 'Unknown date URL.';
			exit;
		}

		$isApp = bobb_dates_init_infusionsoft(get_option('bobb_dates_api_name'), get_option('bobb_dates_api_key'));
		if (!$isApp) {
			echo 'Unable to connect to Infusionsoft.';
			exit;
		}

		date_default_timezone_set('America/New_York');
		$updates = array();
		for ($i = 0; $i < count($data['data_list']['field']); $i++) {
			$field = $data['data_list']['field'][$i];
			$delta = $data['data_list']['delta'][$i];
			$format = $data['data_list']['format'][$i];

			$date = strtotime($delta);
			$updates[$field] = date($format, $date);
		}
		$rc = $isApp->updateContact($contact_id, $updates);

		echo 'ok';
		exit;
	}
}

function bobb_dates_query_vars($vars) {
	$vars[] = 'bobb_dates';
	return $vars;
}

function bobb_dates_get_field_row($fields, $id, $field='', $delta='', $format='', $style='') {
	$deltas = array(
        'days'=>'Day(s)',
        'weeks'=>'Week(s)',
        'months'=>'Month(s)',
        'years'=>'Years(s)',
        'today'=>'Today',
        'next Monday'=>'Next Monday',
        'next Tuesday'=>'Next Tuesday',
        'next Wednesday'=>'Next Wednesday',
        'next Thursday'=>'Next Thursday',
        'next Friday'=>'Next Friday',
        'next Saturday'=>'Next Saturday',
        'next Sunday'=>'Next Sunday',
		);
	$formats = array(
		'm-d-y'=>'05-03-10',
		'm-d-Y'=>'05-03-2010',
		'y-m-d'=>'10-05-03',
		'Y-m-d'=>'2010-05-03 *',
		'n/j/y'=>'5/3/10',
		'n/j/Y'=>'5/3/2010',
		'F j, Y'=>'May 3, 2010',
		'F jS, Y'=>'May 3rd, 2010',
		'l'=>'Saturday',
		'l F jS'=>'Saturday May 3rd',
		);

	if (preg_match('/([\+\-]\d+) (\w+)/', $delta, $matches)) {
		$delta_qty = intval($matches[1]);
		$delta_unit = $matches[2];
	} else if (empty($delta)) {
		$delta_qty = 3;
		$delta_unit = 'days';
	} else {
		$delta_qty = 0;
		$delta_unit = $delta;
	}

	$html = '
	<tr id="bobb-dates-field-'.$id.'" valign="top" '.$style.'>
	  <th scope="row">Infusionsoft Field:</th>
	  <td>
        <select name="bobb_dates_data_list[field][]">';

	foreach ($fields as $fld) {
		$html .= '        <option value="'.$fld.'" '.($field == $fld ? 'selected' : '').'>'.$fld.'</option>'."\n";
	}

	$html .= '
        </select><br/>
        <table>
        <tr><td>Date To Be Entered Into Contact Record:</td>
		<td>
        <input type="text" name="bobb_dates_data_list[delta_qty][]" value="'.$delta_qty.'" style="width:3em" />
        <select name="bobb_dates_data_list[delta_unit][]">';

	foreach ($deltas as $dlt=>$desc) {
		$html .= '        <option value="'.$dlt.'" '.($delta_unit == $dlt ? 'selected' : '').'>'.$desc.'</option>'."\n";
	}

	$html .= '
        </select>
		</td></tr>
        <tr><td>Format Date Like:<br/><em>* use this format for fields that are true date fields.</em></td>
        <td align="right">
        <select name="bobb_dates_data_list[format][]">';

	foreach ($formats as $fmt=>$desc) {
		$html .= '        <option value="'.$fmt.'" '.($format == $fmt ? 'selected' : '').'>'.$desc.'</option>'."\n";
	}


	$html .= '
        </select>
		</td></tr>
        </table>
      </td>
	  <td>
         <input type="button" id="bobb-dates-del-field-'.$id.'" value="Remove Field" class="bobb-dates-del-field" />
      </td>
	</tr>
';

	return $html;
}

function bobb_dates_get_infusionsoft_fields($isApp) {
	include_once('contact_fields.php');

	$customFields = $isApp->dsQuery("DataFormField", 500, 0, array('Name'=>'%'), array('Name', 'GroupId'));
	foreach ($customFields as $field) {
		$contactFields[] = '_'.$field['Name'];
	}
	sort($contactFields);

	return $contactFields;
}

function bobb_dates_get_url_handle($name) {
	$handle = preg_replace('/\W+/', '', preg_replace('/\s+/', '_', strtolower($name)));
	return $handle;
}

function bobb_dates_init_infusionsoft($app_name, $app_key) {
	static $isApps = array();

	if (!isset($isApps[$app_name])) {
		require_once('isdkext.inc');
		$isApps[$app_name] = new iSDKExt();
		if (!$isApps[$app_name]->cfgCon($app_name, $app_key)) {
			$isApps[$app_name] = null;
		}
	}

	return $isApps[$app_name];
}

