<?php
/**
 * Infusionsoft API - PHP SDK
 * 
 * This is a PHP Software Development Kit (SDK) that implements the Infusionsoft XMLRPC API.
 * @package infusionsoft-api
 * @package infusionsoft-api-phpsdk
 * @version 1.4
 * @category infusionsoft,api,sdk
 * @author Justin Morris
 * @link http://www.infusionsoft.com/fusebox/api/field_access.php Infusionsoft API Data Model
 * @link http://www.fuzioncore.com/phpsdk/Doc Infusionsoft API - PHP SDK
 *
 */
require_once(realpath(dirname(__FILE__))."/xmlrpc-2.0/lib/xmlrpc.inc");

/**
 * iSDK
 *
 * Infusionsoft API - PHP SDK Class
 * {@inheritdoc}
 * @package infusionsoft-api
 * @package infusionsoft-api-phpsdk
 */
class iSDK {
////////////////////////////////
//////////CONNECTOR/////////////
////////////////////////////////

###Connect from an entry in the config file###
public function cfgCon($name) {
  $file = fopen(dirname(__FILE__) . "/conn.cfg", "r") or exit("Unable to open file!");
  while(!feof($file))
  {
    $appLine = fgets($file);
    $details[substr($appLine,0,strpos($appLine,":"))] = split(":",$appLine);
  }
  fclose($file);
  if (!empty($details[$name])) {
    if ($details[$name][2]=="i") {
      $this->client = new xmlrpc_client("https://" . $details[$name][1] . ".infusionsoft.com/api/xmlrpc");
    } elseif ($details[$name][2]=="m") {
      $this->client = new xmlrpc_client("https://" . $details[$name][1] . ".mortgageprocrm.com/api/xmlrpc");
    } else {
      return FALSE;
    }
  } else {
    return FALSE;
  }
  
  ###Return Raw PHP Types###
  $this->client->return_type = "phpvals";

  ###Dont bother with certificate verification###
  $this->client->setSSLVerifyPeer(FALSE);

  ###API Key###
  $this->key = $details[$name][3];
  
  if ($this->appEcho("connected?")) {
    return TRUE;
  } else {
	return FALSE;
  }
  
  return TRUE;
}

###Worthless public function, used to validate a connection###
public function appEcho($txt) {
    $carray = array(php_xmlrpc_encode($txt));
    
    $rc = $this->methodCaller("DataService.echo", $carray);
	return $rc == $txt;
}

/////////////////////////////////////////////////////////
////////////////////CONTACT SERVICE//////////////////////
/////////////////////////////////////////////////////////
###public function to add contacts to Infusion - Returns Contact ID###
public function addCon($cMap, $optReason = "") {

    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cMap,array('auto_dates')));
    $conID = $this->methodCaller("ContactService.add",$carray);
    if (!empty($cMap['Email'])) { 
      if ($optReason == "") { $this->optIn($cMap['Email']); } else { $this->optIn($cMap['Email'],$optReason); }
    }
    return $conID;
}

###public function to Update Contacts in Infusion - Returns updated contacts ID###
public function updateCon($cid, $cMap) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($cMap,array('auto_dates')));
    return $this->methodCaller("ContactService.update",$carray);
}

###public function to load a contacts data - Returns a Key/Value array###
public function loadCon($cid, $rFields) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($rFields));
    return $this->methodCaller("ContactService.load",$carray);
}

###public function to add a contact to a group###
public function grpAssign($cid, $gid) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($gid));
    return $this->methodCaller("ContactService.addToGroup",$carray);
}

###public function to remove a contact from a group###
public function grpRemove($cid, $gid) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($gid));
    return $this->methodCaller("ContactService.removeFromGroup",$carray);
}

###public function to add a contact to a campaign###
public function campAssign($cid, $campId) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($campId));
    return $this->methodCaller("ContactService.addToCampaign",$carray);
}

###Returns next step in a campaign###
public function getNextCampaignStep($cid, $campId) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($campId));
    return $this->methodCaller("ContactService.getNextCampaignStep",$carray);
}

###Reschedules a campaign step for a list of contacts###
public function rescheduleCampaignStep($cidList, $campId) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cidList),
                    php_xmlrpc_encode($campId));
    return $this->methodCaller("ContactService.rescheduleCampaignStep",$carray);
}

###public function to remove a contact from a campaign###
public function campRemove($cid, $campId) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($campId));
    return $this->methodCaller("ContactService.removeFromCampaign",$carray);
}

###public function to pause a contacts campaign###
public function campPause($cid, $campId) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($campId));
    return $this->methodCaller("ContactService.pauseCampaign",$carray);
}

###public function to run an Action Sequence###
public function runAS($cid, $aid) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($cid),
                    php_xmlrpc_encode($aid));
    return $this->methodCaller("ContactService.runActionSequence",$carray);
}

/////////////////////////////////////////////////////////
//////////////////////DATA SERVICE///////////////////////
/////////////////////////////////////////////////////////

public function dsAdd($tName,$iMap) {
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($iMap,array('auto_dates')));
    
    return $this->methodCaller("DataService.add",$carray);
}

###public function for DataService.update method###
public function dsUpdate($tName,$id,$iMap) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($id),
                    php_xmlrpc_encode($iMap));
    
    return $this->methodCaller("DataService.update",$carray);
}

###public function for DataService.load method###
public function dsLoad($tName,$id,$rFields) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($id),
                    php_xmlrpc_encode($rFields));
    
    return $this->methodCaller("DataService.load",$carray);
}

###public function for DataService.query method###
public function dsDelete($tName,$id) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($id));
    
    return $this->methodCaller("DataService.delete",$carray);
}

###public function for DataService.findByField method###
public function dsFind($tName,$limit,$page,$field,$value,$rFields) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($limit),
                    php_xmlrpc_encode($page),
                    php_xmlrpc_encode($field),
                    php_xmlrpc_encode($value),
                    php_xmlrpc_encode($rFields));
    
    return $this->methodCaller("DataService.findByField",$carray);
}

###public function for DataService.query method###
public function dsQuery($tName,$limit,$page,$query,$rFields) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($tName),
                    php_xmlrpc_encode($limit),
                    php_xmlrpc_encode($page),
                    php_xmlrpc_encode($query,array('auto_dates')),
                    php_xmlrpc_encode($rFields));
    
    return $this->methodCaller("DataService.query",$carray);
}

###Adds a custom field to Infusionsoft###
public function addCustomField($context,$displayName,$dataType,$groupID) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($context),
                    php_xmlrpc_encode($displayName),
                    php_xmlrpc_encode($dataType),
                    php_xmlrpc_encode($groupID));
    
    return $this->methodCaller("DataService.addCustomField",$carray);
}

###Authenticates a user account in Infusionsoft###
public function authenticateUser($userName,$password) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($userName),
                    php_xmlrpc_encode($password));
    
    return $this->methodCaller("DataService.authenticateUser",$carray);
}

###Updates a custom field###
public function updateCustomField($fieldId, $fieldValues) {
    
    $carray = array(
                    php_xmlrpc_encode($this->key),
                    php_xmlrpc_encode($fieldId),
                    php_xmlrpc_encode($fieldValues));
    return $this->methodCaller("DataService.updateCustomField",$carray);
}
/////////////////////////////////////////////////////////
////////////////////INVOICE SERVICE//////////////////////
/////////////////////////////////////////////////////////

public function manualPmt($invId,$amt,$payDate,$payType,$payDesc,$bypassComm) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($invId),
        php_xmlrpc_encode($amt),
        php_xmlrpc_encode($payDate,array('auto_dates')),
        php_xmlrpc_encode($payType),
        php_xmlrpc_encode($payDesc),
        php_xmlrpc_encode($bypassComm));
    return $this->methodCaller("InvoiceService.addManualPayment",$carray);
}

###public function to Override Order Commisions - InvoiceService.addOrderCommissionOverride###
public function commOverride($invId,$affId,$prodId,$percentage,$amt,$payType,$desc,$date) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($invId),
        php_xmlrpc_encode($prodId),
        php_xmlrpc_encode($percentage),
        php_xmlrpc_encode($amt),
        php_xmlrpc_encode($payType),
        php_xmlrpc_encode($desc),
        php_xmlrpc_encode($date,array('auto_dates')));
    
    return $this->methodCaller("InvoiceService.addOrderCommissionOverride",$carray);
}

###public function to add an item to an order - InvoiceService.addOrderItem###
public function addOrderItem($ordId,$prodId,$type,$price,$qty,$Desc,$Notes) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($ordId),
        php_xmlrpc_encode($prodId),
        php_xmlrpc_encode($type),
        php_xmlrpc_encode($price),
        php_xmlrpc_encode($qty),
        php_xmlrpc_encode($Desc),
        php_xmlrpc_encode($notes));
    
    return $this->methodCaller("InvoiceService.addOrderItem",$carray);
}

###public function to add payment plans to orders - InvoiceService.addPaymentPlan###
public function payPlan($ordId,$aCharge,$ccId,$merchId,$retry,$retryAmt,$initialPmt,$initialPmtDate,$planStartDate,$numPmts,$pmtDays) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($ordId),
        php_xmlrpc_encode($aCharge),
        php_xmlrpc_encode($ccId),
        php_xmlrpc_encode($merchId),
        php_xmlrpc_encode($retry),
        php_xmlrpc_encode($retryAmt),
        php_xmlrpc_encode($initialPmt),
        php_xmlrpc_encode($initialPmtDate,array('auto_dates')),
        php_xmlrpc_encode($planStartDate,array('auto_dates')),
        php_xmlrpc_encode($numPmts),
        php_xmlrpc_encode($pmtDays));
    
    return $this->methodCaller("InvoiceService.addPaymentPlan",$carray);
}

###public function to Override Recurring Order Commisions - InvoiceService.addOrderCommissionOverride###
public function recurringCommOverride($recId,$affId,$amt,$payType,$desc) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($recId),
        php_xmlrpc_encode($affId),
        php_xmlrpc_encode($amt),
        php_xmlrpc_encode($payType),
        php_xmlrpc_encode($desc));
    
    return $this->methodCaller("InvoiceService.addRecurringCommissionOverride",$carray);
}

###public function to add a recurring order - InvoiceService.addRecurringOrder###

public function addRecurring($cid,$allowDup,$progId,$merchId,$ccId,$affId,$daysToCharge) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($cid),
        php_xmlrpc_encode($allowDup),
        php_xmlrpc_encode($progId),
        php_xmlrpc_encode($merchId),
        php_xmlrpc_encode($ccId),
        php_xmlrpc_encode($affId),
        php_xmlrpc_encode($daysToCharge));
    return $this->methodCaller("InvoiceService.addRecurringOrder",$carray);
}

###public function to get the Amount owed on an invoice - InvoiceService.calculateAmountOwed###
public function amtOwed($invId) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($invId));
    
    return $this->methodCaller("InvoiceService.calculateAmountOwed",$carray);
}

###Find the id of the invoice attached to a one-time order###
public function getInvoiceId($orderId) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($orderId));
    
    return $this->methodCaller("InvoiceService.getInvoiceId",$carray);
}

###public function to charge invoices - InvoiceService.chargeInvoice###
public function chargeInvoice($invId,$notes,$ccId,$merchId,$bypassComm) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($invId),
        php_xmlrpc_encode($notes),
        php_xmlrpc_encode($ccId),
        php_xmlrpc_encode($merchId),
        php_xmlrpc_encode($bypassComm));
    
    return $this->methodCaller("InvoiceService.chargeInvoice",$carray);
}

###public function to create blank orders - InvoiceService.createBlankOrder###
public function blankOrder($conId,$desc,$oDate,$leadAff,$saleAff) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($conId),
        php_xmlrpc_encode($desc),
        php_xmlrpc_encode($oDate,array('auto_dates')),
        php_xmlrpc_encode($leadAff),
        php_xmlrpc_encode($saleAff));
    
    return $this->methodCaller("InvoiceService.createBlankOrder",$carray);
}

###public function to create an invioce for recurring orders - InvoiceService.createInvoiceForRecurring###
public function recurringInvoice($rid) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($rid));
    
    return $this->methodCaller("InvoiceService.createInvoiceForRecurring",$carray);
}

###public function to locate creditcards based on the last 4 digits - InvoiceService.locateExistingCard###
public function locateCard($cid,$last4) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($cid),
        php_xmlrpc_encode($last4));
    
    return $this->methodCaller("InvoiceService.locateExistingCard",$carray);
}

###public function to Validate Credit Cards - InvoiceService.validateCreditCard###
###This public function will take a CC ID or a CC Map###
public function validateCard($ccId) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($ccId));
    
    return $this->methodCaller("InvoiceService.validateCreditCard",$carray);
}

#############################
##### API EMAIL SERVICE #####
#############################

###This function will attach an email to a contacts email history###
public function attachEmail($cId, $fromName, $fromAddress, $toAddress, $ccAddresses,
			    $bccAddresses, $contentType, $subject, $htmlBody, $txtBody,
			    $header, $strRecvdDate, $strSentDate) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($cId),
	php_xmlrpc_encode($fromName),
	php_xmlrpc_encode($fromAddress),
	php_xmlrpc_encode($toAddress),
	php_xmlrpc_encode($ccAddresses),
	php_xmlrpc_encode($bccAddresses),
	php_xmlrpc_encode($contentType),
	php_xmlrpc_encode($subject),
	php_xmlrpc_encode($htmlBody),
	php_xmlrpc_encode($txtBody),
	php_xmlrpc_encode($header),
	php_xmlrpc_encode($strRecvdDate),
	php_xmlrpc_encode($strSentDate));
    
    return $this->methodCaller("APIEmailService.attachEmail",$carray);
}

###This function will send an email to an array contacts###
public function sendEmail($conList, $fromAddress, $toAddress, $ccAddresses, $bccAddresses, $contentType, $subject, $htmlBody, $txtBody) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($conList),
	php_xmlrpc_encode($fromAddress),
	php_xmlrpc_encode($toAddress),
	php_xmlrpc_encode($ccAddresses),
	php_xmlrpc_encode($bccAddresses),
	php_xmlrpc_encode($contentType),
	php_xmlrpc_encode($subject),
	php_xmlrpc_encode($htmlBody),
	php_xmlrpc_encode($txtBody));
    
    return $this->methodCaller("APIEmailService.sendEmail",$carray);
}


public function createEmailTemplate($title, $userID, $fromAddress, $toAddress, $ccAddresses, $bccAddresses, $contentType, $subject, $htmlBody, $txtBody) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($title),
        php_xmlrpc_encode($userID),
        php_xmlrpc_encode($fromAddress),
        php_xmlrpc_encode($toAddress),
        php_xmlrpc_encode($ccAddresses),
        php_xmlrpc_encode($bccAddresses),
        php_xmlrpc_encode($contentType),
        php_xmlrpc_encode($subject),
        php_xmlrpc_encode($htmlBody),
        php_xmlrpc_encode($txtBody));
    
    return $this->methodCaller("APIEmailService.createEmailTemplate",$carray);
}

public function addEmailTemplate($title, $categories, $fromAddress, $toAddress, $ccAddress, $bccAddress, $subject, $textBody, $htmlBody, $contentType, $mergeContext) {

    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($title),
        php_xmlrpc_encode($categories),
        php_xmlrpc_encode($fromAddress),
        php_xmlrpc_encode($toAddress),
        php_xmlrpc_encode($ccAddresses),
        php_xmlrpc_encode($bccAddresses),
        php_xmlrpc_encode($subject),
        php_xmlrpc_encode($textBody),
        php_xmlrpc_encode($htmlBody),
        php_xmlrpc_encode($contentType),
		php_xmlrpc_encode($mergeContext));
    
    return $this->methodCaller("APIEmailService.addEmailTemplate",$carray);
}

public function getEmailTemplate($templateId) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode(intval($templateId)));
    
    return $this->methodCaller("APIEmailService.getEmailTemplate",$carray);
}

###Functions to opt people in/out.###
###Note that Opt-In will only work on "non-marketable contacts not opted out people.###
public function optIn($email, $reason='API Opt In') {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($email),
        php_xmlrpc_encode($reason));
    
    return $this->methodCaller("APIEmailService.optIn",$carray);
}

public function optOut($email, $reason='API Opt Out') {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($email),
        php_xmlrpc_encode($reason));
    
    return $this->methodCaller("APIEmailService.optOut",$carray);
}

////////////////////////////////////////////////////////
////////////////AFFILIATE SYSTEM FUNCTIONS//////////////
////////////////////////////////////////////////////////

###This function will return all claw backs in a date range###
public function affClawbacks($affId, $startDate, $endDate) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($affId),
        php_xmlrpc_encode($startDate),
        php_xmlrpc_encode($endDate));
    
    return $this->methodCaller("APIAffiliateService.affClawbacks",$carray);
}

###This function will return all commissions in a date range###
public function affCommissions($affId, $startDate, $endDate) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($affId),
        php_xmlrpc_encode($startDate),
        php_xmlrpc_encode($endDate));
    
    return $this->methodCaller("APIAffiliateService.affCommissions",$carray);
}

###This function will return all payouts in a date range###
public function affPayouts($affId, $startDate, $endDate) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($affId),
        php_xmlrpc_encode($startDate),
        php_xmlrpc_encode($endDate));
    
    return $this->methodCaller("APIAffiliateService.affPayouts",$carray);
}

###Returns a list with each row representing a single affiliates totals represented by a map with key (one of the names above, and value being the total for that variable)###
public function affRunningTotals($affList) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($affList));
    
    return $this->methodCaller("APIAffiliateService.affRunningTotals",$carray);
}

###This function will return how much the specified affiliates are owed###
public function affSummary($affList, $startDate, $endDate) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($affList),
        php_xmlrpc_encode($startDate),
        php_xmlrpc_encode($endDate));
    
    return $this->methodCaller("APIAffiliateService.affSummary",$carray);
}

////////////////////////////////////////////////////////
//////////////// TICKET SYSTEM FUNCTIONS ///////////////
////////////////////////////////////////////////////////

###This function Adds move notes to existing tickets###
public function addMoveNotes($ticketList, $moveNotes, $moveToStageId, $notifyIds) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($ticketList),
        php_xmlrpc_encode($moveNotes),
        php_xmlrpc_encode($moveToStageId),
        php_xmlrpc_encode($notifyIds));
    
    return $this->methodCaller("ServiceCallService.addMoveNotes",$carray);
}

###This function Adds move notes to existing tickets###
public function moveTicketStage($ticketID, $ticketStage, $moveNotes, $notifyIds) {
    
    $carray = array(
        php_xmlrpc_encode($this->key),
        php_xmlrpc_encode($ticketID),
        php_xmlrpc_encode($ticketStage),
        php_xmlrpc_encode($moveNotes),
        php_xmlrpc_encode($notifyIds));
    
    return $this->methodCaller("ServiceCallService.moveTicketStage",$carray);
}

/////////////////////////////////////////////////////////
////////////////ADDITIONAL public functionS//////////////
/////////////////////////////////////////////////////////

###public function to return properly formatted dates.
public function infuDate($dateStr) {
    $dArray=date_parse($dateStr);
    if ($dArray['error_count']<1) {
        $tStamp = mktime($dArray['hour'],$dArray['minute'],$dArray['second'],$dArray['month'],$dArray['day'],$dArray['year']);
        return date('Ymd\TH:i:s',$tStamp);
    } else {
        foreach ($dArray['errors'] as $err) {
            echo "ERROR: " . $err . "<br />";
        }
        die("The above errors prevented the application from executing properly.");
    }
}
}
?>
