<?php 
# vim:ts=4:nowrap

require_once(realpath(dirname(__FILE__))."/isdk.php");

class iSDKExt extends iSDK {

	/* cfgCon
     *
     * Custom version of cfgCon that takes the app name (unused) and key vs. using conn.cfg file.
     */
	public function cfgCon($appName, $appKey) {
        $this->client = new xmlrpc_client("https://" . $appName . "/api/xmlrpc");

  		###Return Raw PHP Types###
  		$this->client->return_type = "phpvals";

		###Dont bother with certificate verification###
		$this->client->setSSLVerifyPeer(FALSE);

		###API Key###
		$this->key = $appKey;
  
  		return $this->appEcho('connected?');
	}

	/* loadContact
	 *
	 * Loads a single contact given a contact id.
     */
	public function loadContact($contactId, $fields) {
		$data = $this->loadCon(intval($contactId), $fields);
		if (isset($data['error'])) {
			die('Error: '.$data['error']['message']);
		}

		$data['Id'] = $contactId;

		return $data;
	}

	/* updateContact
	 *
	 * Updates a single contact given a contact id and the fields/values to update.
     */
	public function updateContact($contactId, $fields) {
		$data = $this->updateCon(intval($contactId), $fields);
		if (isset($data['error'])) {
			error_log('Error: '.$data['error']['message']);
			return false;
		}

		return true;
	}

	/* methodCaller
	 *
	 * Override iSDK::methodCaller so we can return errors rather than die().
	 * @param string $service xmlrpc service to call
	 * @param mixed $callArray xmlrpcmsg call params
	 * @returns mixed Array containing result.
	 */
	public function methodCaller($service,$callArray) {
		$call = new xmlrpcmsg($service, $callArray);
		$result = $this->client->send($call);
		if(!$result->faultCode()) {
			return $result->value();
		} else {
			return array(
				'error' => array(
					'code'    => $result->faultCode(),
					'message' => $result->faultString(),
					'service' => $service,
					'params'  => $callArray
					)
				);
		}
	}

}
